Define CONF_WORD = 0x3f63  'external clock
Define CONF_WORD_2 = 0x3ffc

'Define CONF_WORD = 0x3f62 ' HS clock Osc

'Rs232 to I2C and others controler Paul McMahon VK3DIP 10/01/2009
'set up for a 16f88

'Commands from PC

'W aa,hh,hh,hh,hh,..<CR><LF>
'Write to I2C device (ie Tuner)at slave address aa
'at least 1 up to decimal 17 bytes hh, first hh can be ss a subaddress if required
'response to PC from write
'OK

'R aa<CR><LF>
'Read one byte from I2C device at slave address aa
'response to PC from Read
'Raa=hh<CR><LF>

'R aa,nn<CR><LF>
'Read nn (01 < nn < ff )bytes from from I2C device at slave address aa
'response to PC from Read
'Raa=hh,hh,hh..<CR><LF>

'R aa,ss,nn<CR><LF>
'Read nn (01 < nn < ff )bytes from from I2C device
'at slave address aa, subaddress ss, all in lowercase hex
'response to PC from Read
'Raass=hh,hh,hh..<CR><LF>

'R aa,ssss,nn<CR><LF>
'Read nn (01 < nn < ff )bytes from from I2C device
'at slave address aa, subaddress ssss, all in lowercase hex
'response to PC from Read
'Raassss=hh,hh,hh..<CR><LF>


'D hh,hh,hh,hh,hh<CR><LF>
'Write to DDS device (ie AD9850)up to 40 bits commas optional
'last typed hh is done first bit 0 sent first then bit 1  to bit 7, then next hh ie right to left
'response to PC from write
'OK

'V 0p,nn<CR><LF>
'Get 10 bit analog voltage from port 0p, 0p = 00,01,02 (03 = 00&01&02)
'if nn present repeat for hex value nm times, if not present do once
'Response to PC
'V0p=hhhh<CR><LF>

'B 0b<CR><LF>
'Set digital bits out for RB7,RB3,RA4  0p = 00 to 07
'00 => RB7,RB3,RA4  = 0,0,0 :  07 => RB7,RB3,RA4  = 1,1,1
'Response to PC
'OK

'P hh <CR><LF>
'PWM out ( on RB0) 8 bit duty cycle at approx 8 Khz
'hh is 2 hex bytes giving 0 - 255 (00 To FF) levels
'00 is off 01 - FF is on.
'Response to PC
'OK

'F gt<CR><LF>
'Get 3 byte hex count from T1CLK input for gt  01 = 0.1 second, 10 = 1 second
'Response to PC
'Fgt=hhhhhh<CR><LF>

'C 0p,dd<CR><LF>
'wait for sda pin to from high to low then
'get series of 10 Bit analog voltages from port 0p, 0p = 00, 01, 02, Not 03
'at delay 0.1 plus dd ( 00 to ff) * 0.1 ms between each sample.
'fill buffer with analog readings hhhh and then send to PC.
'Response to PC
'C0p,dd<CR><LF>
'[<CR><LF>
'hhhh<CR><LF>
'hhhh<CR><LF>
'hhhh<CR><LF>
'hhhh<CR><LF>
'.....
'hhhh<CR><LF>
'hhhh<CR><LF>
']<CR><LF>



'? <CR><LF>
'Get Version Info
'Response to PC
'Version Info


'repeat last command
'<CR><LF>

'In all cases spaces, commas etc. treated as white space and ignored
'All hex alpha characters must be lower case, all commands are upper case.
'aa is a ascii hex byte defining the I2C slave address
'hh is a hex byte 00 to ff ( lower case only)
'0p = 00,01,02 for relevant port
'0p = 00,01,02,03,04,05,06,07 on RB7,RB3,RA4 as three least significant bits

'hhhh = four digit hex number 0000 to 03ff coresponding to 0V to +Ref on A.3
'hhhhhh = 6 digit hex number 000000 to ffffff coresponding to count for selected gate time.


Const hinib = %11110000  'masks for getting nibbles
Const lonib = %00001111
Symbol sda = PORTB.1  'use RB1 as SDA pin and Capture trigger
Symbol scl = PORTB.4  'use RB4 as SCL pin
Symbol gend1 = PORTA.4  'Generic digital out LSB, also DDS W_CLK
Symbol gend2 = PORTB.3  'Generic digital middle bit, also DDS Data
Symbol gend3 = PORTB.7  'Generic digital MSB, also DDS FQ_UD


Dim sladdr As Byte  'I2C Slave address
Dim dnib As Bit  'nibble we are up to 1 = hi, 0 = lo
Dim mycmd As Byte  'command to be executed
'0 = none, 1 = I2C write, 2 = i2creadread, 3 = Analog read, 4 = ?,
'5 = PWM, 6 = frequency, 7 = digital out, 8 = DDS out, 9 = Capture
Dim dcnt As Byte  'count of data bytes max should be 49, 0 = no data
Dim rdata As Byte  'General read in of one byte
Dim myhex As Byte  'common hex value
Dim myparam As Byte
Dim mysubval As Byte
Dim myp2 As Byte
Dim mycnt As Byte
Dim mycnt2 As Byte
Dim mychar As Byte  'char in from serial etc. port
Dim myvolts As Word  'read in 10 bit voltage
Dim myd1 As Byte
Dim myd2 As Byte
Dim myd3 As Byte
Dim myb2 As Byte
Dim myb1 As Byte
Dim myb0 As Byte
Dim mydbit As Byte  'passed DiBit first 2 bits only used
Dim mybyte As Byte  'passed byte
Dim tempstr As Byte  'temporary store on way in or out of main store
Dim tempnxt As Byte  'location of next DiBit in tempstr ie. 0,2,4,6
Const maxb0 = 49  'maximum index of block 0
Const maxb1 = 79  'maximum index of block 1
Const maxb2 = 95  'maximum index of block 2
Const maxb3 = 95  'maximum index of block 3
Dim mymax(4) As Byte  'maximum of block 0 - 3
Dim bptr As Byte  'current block pointer
Dim aptr As Byte  'current address in block pointer
Dim mydata(50) As Byte  'data bytes to send to the tuner, plus block 0
Dim mydata1(80) As Byte  'block 1
Dim mydata2(96) As Byte  'block 2
Dim mydata3(96) As Byte  'block 3

Define I2CREAD_DELAYUS = 80  '0 and 20 give 1/2 true range, 50 OK
'Define I2CCLOCK_STRETCH = 2 - doesn't seem to be needed with phillips tuners
TRISA = %11101111  'set all PORTA pins as inputs except A4 to use as D0 out
TRISB = %01010110  'set PORTB pins
ANSEL = %00001111  'set analog inputs
'ADCON0 = 0xc0  ' don't set ADCON0 directly use compiler defines
Define ADC_CLOCK = 3  'Use A/D with internal RC clock
Define ADC_SAMPLEUS = 20  'default value is 20
ADCON1 = %10100000  'set right shifted, clock divide disabled, Vref+ AVss

Hseropen 57600

WaitMs 1000  'just to let things settle down

mycmd = 0  'initialize some variables
dnib = 1
dcnt = 0
mymax(0) = maxb0  'set maximums
mymax(1) = maxb1
mymax(2) = maxb2
mymax(3) = maxb3
bptr = 0  'current block pointer
aptr = 0  'current address in block pointer
tempstr = 0
tempnxt = 0

loop:

	If RCSTA.OERR Then  'check for serial overrun and clear if found
		RCSTA.CREN = 0
		RCSTA.CREN = 1
	Else
		Hserget mychar  'get the character if present
		If mychar > 0 Then  'see if we got one
			Select Case mychar  'parse it
				Case 0x57  'W
					dcnt = 0  'new command reset data
					dnib = 1
					mycmd = 1  'set cmd to i2c write
				Case 0x52  'R
					dcnt = 0  'new command reset data
					dnib = 1
					mycmd = 2  'set cmd to i2c read
				Case 0x56  'V
					dcnt = 0  'new command reset data
					dnib = 1
					mycmd = 3  'set cmd to Analog in
				Case 0x3f  '?
					dcnt = 0  'new command reset data
					dnib = 1
					mycmd = 4  'set cmd to help
				Case 0x50  'P
					dcnt = 0  'new command reset data
					dnib = 1
					mycmd = 5  'set cmd to PWM
				Case 0x46  'F
					dcnt = 0  'new command reset data
					dnib = 1
					mycmd = 6  'set cmd to frequency
				Case 0x42  'B
					dcnt = 0  'new command reset data
					dnib = 1
					mycmd = 7  'set cmd to frequency
				Case 0x44  'D
					dcnt = 0  'new command reset data
					dnib = 1
					mycmd = 8  'set cmd to write DDS
				Case 0x43  'C
					dcnt = 0  'new command reset data
					dnib = 1
					mycmd = 9  'set cmd to capture data


				Case 0x0a  '<lf> end of input from pc so execute command
					Hserout 0x13  'send XOFF to stop more data frpm PC while working
					Select Case mycmd
						Case 1  'it is a I2C write
							If dcnt > 1 Then  'we at least have the address and one byte to write
								sladdr = mydata(0)
								sladdr.0 = 0  'just to be sure it is a write address
								I2CPrepare sda, scl
								I2CStart
								I2CSend sladdr
								WaitUs 80  'match the delay with the read
								For mycnt = 2 To dcnt
									rdata = mycnt - 1
									I2CSend mydata(rdata)
								Next mycnt
								I2CStop
							Else  'No data
								Gosub cmderror
							Endif
							dcnt = 0  'all sent
							Hserout "OK", CrLf
						Case 2  'it is a I2C read
							Select Case dcnt
								Case 1  'just the address ie. get one byte
									sladdr = mydata(0)  'first data byte will be the address
									sladdr.0 = 1  'ensure lsb = 1 to indicate read
									rdata = 0  'clear it before reading
									I2CRead1 sda, scl, sladdr, rdata  'Read from the tuner
									Hserout "R"
									mysubval = sladdr
									Gosub writebyte
									Hserout "="  'send back to the PC the result
									mysubval = rdata
									Gosub writebyte
									Hserout CrLf
								Case 2  'the address and a number of bytes
									If mydata(1) > 0 Then  'don't get 0 bytes
										sladdr = mydata(0)  'first data byte will be the address
										sladdr.0 = 1  'ensure lsb = 1 to indicate read
										Hserout "R"
										I2CPrepare sda, scl
										I2CStart
										I2CSend sladdr  'the address
										mysubval = sladdr  'this also is a delay between i2c bytes
										Gosub writebyte
										Hserout "="  'send back to the PC the result
										For mycnt = 2 To mydata(1)  'get number of bytes less 1
											I2CReceiveAck mysubval
											Gosub writebyte
											Hserout ","
										Next mycnt
										I2CRecN mysubval  'get last byte with a nak
										Gosub writebyte
										Hserout CrLf
										I2CStop
									Else
										Gosub cmderror
									Endif
								Case 3  'the address, byte subaddress and a number of bytes
									If mydata(2) > 0 Then  'don't get 0 bytes
										sladdr = mydata(0)  'first data byte will be the address
										Hserout "R"
										I2CPrepare sda, scl
										I2CStart
										sladdr.0 = 0  'ensure lsb = 0 to indicate subaddress to follow
										I2CSend sladdr  'the address
										mysubval = sladdr  'this also is a delay between i2c bytes
										Gosub writebyte
										I2CSend mydata(1)  'the sub address
										mysubval = mydata(1)  'this also is a delay between i2c bytes
										Gosub writebyte
										sladdr.0 = 1  'ensure lsb = 1 to indicate read
										I2CStart
										I2CSend sladdr  'the address with read bit
										Hserout "="  'send back to the PC the result
										For mycnt = 2 To mydata(2)  'get number of bytes less 1
											I2CReceiveAck mysubval
											Gosub writebyte
											Hserout ","
										Next mycnt
										I2CRecN mysubval  'get last byte with a nak
										Gosub writebyte
										Hserout CrLf
										I2CStop
									Else
										Gosub cmderror
									Endif
								Case 4  'the address, word subaddress and a number of bytes
									If mydata(3) > 0 Then  'don't get 0 bytes
										sladdr = mydata(0)  'first data byte will be the address
										Hserout "R"
										I2CPrepare sda, scl
										I2CStart
										sladdr.0 = 0  'ensure lsb = 0 to indicate subaddress to follow
										I2CSend sladdr  'the address
										mysubval = sladdr  'this also is a delay between i2c bytes
										Gosub writebyte
										I2CSend mydata(1)  'the first byte of subaddress
										mysubval = mydata(1)  'this also is a delay between i2c bytes
										Gosub writebyte
										I2CSend mydata(2)  'the second byte of subaddress
										mysubval = mydata(2)  'this also is a delay between i2c bytes
										Gosub writebyte
										I2CStop
										sladdr.0 = 1  'ensure lsb = 1 to indicate read
										I2CStart
										I2CSend sladdr  'the address with read bit
										Hserout "="  'send back to the PC the result
										For mycnt = 2 To mydata(3)  'get number of bytes less 1
											I2CReceiveAck mysubval
											Gosub writebyte
											Hserout ","
										Next mycnt
										I2CRecN mysubval  'get last byte with a nak
										Gosub writebyte
										Hserout CrLf
										I2CStop
									Else
										Gosub cmderror
									Endif
									Case Else  'too much or too little
									Gosub cmderror
							EndSelect
						Case 3  'it is 10 bit Analog Voltage read
							sladdr = mydata(0)  'get the port to read
							Select Case dcnt
								Case < 1
									Gosub cmderror
								Case 1  'if there is just a port then do it
									Gosub dovoltsall
								Case 2  'if there is a port and a repeat count do it many times
									For mycnt = 1 To mydata(1)
										Gosub dovoltsall
									Next mycnt
								Case Else
									Gosub cmderror
							EndSelect
						Case 4  'it is a help "?"
								Hserout "VK3DIP Simple Serial IO V4.1E", CrLf  'send back to the PC the result
						Case 5  'it is PWM out
							If dcnt > 0 Then  'we at least have the duty cycle
								rdata = mydata(0)
								If rdata > 0 Then
									PWMon 1, 9  '8 bit res 39.06Khz at 10M xtal
									PWMduty 1, mydata(0)
								Else
									PWMoff 1
								Endif
								Hserout "OK", CrLf
							Else  'no data
								Gosub cmderror
							Endif
						Case 6  'it is Frequency
							If dcnt > 0 Then  'we at least have something for the gate time
								rdata = mydata(0)
								'To tweek gate times vary myd1, each myd1 adds 11 instrutions
								Select Case rdata
								Case 0x01  'Gate time = 0.1 seconds
									Gosub getfreq2
								Case 0x10  'Gate time = 1 seconds
									Gosub getfreq1
								Case Else  'default to 1 second
									Gosub getfreq1
								EndSelect
										Hserout "F"
										mysubval = rdata
										Gosub writebyte
										Hserout "="  'send back to the PC the result
										mysubval = myb2
										Gosub writebyte
										mysubval = myb1
										Gosub writebyte
										mysubval = myb0
										Gosub writebyte
										Hserout CrLf
							Else  'no data
								Gosub cmderror
							Endif

						Case 7  'it is digital Bits out on RB7,RB3,RA4
							If dcnt > 0 Then  'we at least have the byte out
								rdata = mydata(0)
								If rdata < 8 Then
									Gosub bitout
									Hserout "OK", CrLf
								Else
									Gosub cmderror
								Endif
							Else  'no data
								Gosub cmderror
							Endif
						Case 8  'it is DDS style write
							If dcnt > 0 Then  'we at least have one byte to send
									sladdr = dcnt - 1
									gend1 = 0  'clock low to start
									gend3 = 0  'FQ_UD to low to start
									'send as many bytes as we have got bit at a time
									'Start with last entered byte first
									'sendbyte sends bit 0 first
									'ie bits will be sent right to left
									For mycnt = sladdr To 0 Step -1
										myparam = mydata(mycnt)
										Gosub sendbyte
									Next mycnt
									gend3 = 1  'toggle FQ_UD to latch the data.
									WaitUs 10
									gend3 = 0
									WaitUs 10
									Hserout "OK", CrLf

							Else
									Gosub cmderror
							Endif
						Case 9  'it is a 10 bit Analog Voltage capture
							
								If dcnt = 2 Then  'All ok got port and delay
									sladdr = mydata(0)  'get the port to read
									myp2 = mydata(1)  'save the required delay
									If sladdr < 2 Then  'all OK valid port
										bptr = 0  'set to begining of buffers
										aptr = 0
										tempstr = 0
										tempnxt = 0
										mycmd = 0  'we are using the buffer so stop reuse
										TRISB.1 = 1  'make sure its an input
										While Not sda  'wait till sda is high
										Wend  'these will wait for a high to low transition
										While sda  'now wait till sda line goes low
										Wend  'may hang for ever.
										While bptr <> 9  'keep going till buffer full
											Adcin sladdr, myvolts
											mybyte = myvolts.LB
											Gosub addbyte
											mydbit = myvolts.HB
											Gosub adddb
											For mycnt = 0 To myp2  'Approx 100us steps
												WaitUs 100  'wait for requested intersample delay
											Next mycnt
										Wend
										'got them all send them out
										bptr = 0  'set to begining of buffers
										aptr = 0
										tempstr = 0
										tempnxt = 0
										Hserout "C"
										mysubval = sladdr
										Gosub writebyte
										Hserout ","  'send back to the PC the result
										mysubval = myp2
										Gosub writebyte
										Hserout CrLf
										Hserout "["  'send back to the PC the result
										Hserout CrLf
										While bptr <> 9  'keep going till buffer empty
											Gosub getbyte
											Gosub getdb
											mysubval = mydbit
											Gosub writebyte
											mysubval = mybyte
											Gosub writebyte
											Hserout CrLf
										Wend
										Hserout "]"
										Hserout CrLf
									Else  'invalid port
										Gosub cmderror
									Endif

								Else  'otherwise it is an error
									Gosub cmderror
								Endif
						
					EndSelect  'do nothing for unknown commands
					Hserout 0x11  'send XON to say all done to the PC and start sending.
				Case > 0x66  'everything above f not already trapped is an error
					Gosub cmderror
				Case > 0x60  'assume a and up ( > ')not already trapped is a Hex charin
					myparam = mychar - 87
					Gosub setdata
				Case > 0x39  'everything above 9 not already trapped is an error
					Gosub cmderror
				Case > 0x2f  'assume anything left 0 and up ( > /) is a hex numin
					myparam = mychar - 48
					Gosub setdata
			EndSelect  'anything else left ie. below 0 is whitespace and ignored.
		Endif
	Endif
Goto loop

End                                               
tohex:
If myparam < 10 Then
	myhex = 0x30 + myparam
Else
	myhex = 0x57 + myparam
Endif
Return                                            
writebyte:
	myparam = mysubval And hinib
	myparam = ShiftRight(myparam, 4)  'get high nibble
	Gosub tohex
	Hserout myhex
	myparam = mysubval And lonib  'get low nibble of high byte
	Gosub tohex
	Hserout myhex
Return                                            
dovolts:
	Adcin myp2, myvolts
	Hserout "V"
	mysubval = myp2
	Gosub writebyte
	Hserout "="  'send back to the PC the result
	mysubval = myvolts.HB
	Gosub writebyte
	mysubval = myvolts.LB
	Gosub writebyte
	Hserout CrLf
Return                                            
dovoltsall:
	Select Case sladdr
		Case < 0x03
			myp2 = sladdr
			Gosub dovolts
		Case 0x03  'we want them all
			myp2 = 0x00
			Gosub dovolts
			myp2 = 0x01
			Gosub dovolts
			myp2 = 0x02
			Gosub dovolts
		Case Else
			Gosub cmderror
	EndSelect
Return                                            
setdata:  'add the received nibble to the appropriate data byte.
If dcnt > maxb0 Then  'if we get more than max siz of first databuffer start again and ignore data
	Gosub cmderror
Else
	If dnib Then  'do high nibble
		mydata(dcnt) = mydata(dcnt) And lonib  'save lo nibble of data, clear hi
		myparam = ShiftLeft(myparam, 4)  'move param to hi nibble posn
		myparam = myparam And hinib  'ensure only hi nibble used
		mydata(dcnt) = mydata(dcnt) + myparam  'add hi nibble to data
		dnib = 0  'lo nibble next
	Else  'do lo nibble
		mydata(dcnt) = mydata(dcnt) And hinib  'save hi nibble of data, clear lo
		myparam = myparam And lonib  'ensure only lo nibble used.
		mydata(dcnt) = mydata(dcnt) + myparam  'add lo nibble to data
		dcnt = dcnt + 1  'finished this data byte ready for next
		dnib = 1  'start at hi nibble
	Endif
Endif
Return                                            
cmderror:
	mycmd = 0
	dnib = 1
	dcnt = 0
	Hserout "Command Error", CrLf
Return                                            
bitout:  'set bits out for generic digital bits out.
Select Case rdata
	Case 0
		gend1 = 0
		gend2 = 0
		gend3 = 0
	Case 1
		gend1 = 1
		gend2 = 0
		gend3 = 0
	Case 2
		gend1 = 0
		gend2 = 1
		gend3 = 0
	Case 3
		gend1 = 1
		gend2 = 1
		gend3 = 0
	Case 4
		gend1 = 0
		gend2 = 0
		gend3 = 1
	Case 5
		gend1 = 1
		gend2 = 0
		gend3 = 1
	Case 6
		gend1 = 0
		gend2 = 1
		gend3 = 1
	Case 7
		gend1 = 1
		gend2 = 1
		gend3 = 1
EndSelect
Return                                            
getfreq1:
'Uses a one second gate ie 2500000 counts

	myd1 = 180  'D1 changes 10 counts per number ie subtracting one from D1 subtracts 10 instruction cycles
	myd2 = 210
	myd3 = 4
	T1CON = 0  'stop timer
	INTCON = 0  'disable interupts
	PIR1 = 0
	PIE1 = 0
	myb2 = 0  'clear counters
	myb1 = 0  'clear counters
	myb0 = 0  'clear counters
	TMR1H = 0
	TMR1L = 0
	T1CON = 0x06  'set for T1CLK in for async counter no prescale, gate currently shut
ASM:        movlw 0x1F  'Set to bank 00
ASM:        andwf status,f
ASM:        bsf 0x10,0  'open gate TMR1 started
ASM:d1loop1: decfsz myd1,F  'inner delay loop first time use initial value all others use 0=256
ASM:        goto mycchk1
ASM:        goto d2loop1
ASM:mycchk1: btfss 0x0c,0  'look for an overflow
ASM:        goto mychkf1
ASM:        bcf 0x0c,0  'clear the overflow
ASM:        incf myb2,F  'increment the overflow counter
ASM:        nop
ASM:        goto d1loop1
ASM:mychkf1: nop  'not OF so equalize delay
ASM:        nop
ASM:        goto d1loop1
ASM:d2loop1: decfsz myd2,F  'second level delay loop first time use initial value all others use 0=256
ASM:        goto d1loop1
ASM:        decfsz myd3,F  'outer delay loop uses initial value
ASM:        goto d1loop1
ASM:        nop  'fine tuning
ASM:        nop
ASM:        nop
ASM:        nop
ASM:        nop
ASM:        nop
ASM:        bcf 0x10,0  'close gate, counter stopped
ASM:        btfss 0x0c,0  'check for a count/OF coming in while busy elsewhere
ASM:        goto doneit1
ASM:        incf myb2,F
ASM:doneit1: bcf 0x0c,0
myb1 = TMR1H
myb0 = TMR1L
Return                                            
getfreq2:
'Uses a point one second gate ie 250000 counts
	myd1 = 197  'D1 changes 10 counts per number ie subtracting one from D1 subtracts 10 instruction cycles
	myd2 = 98
	myd3 = 1
	T1CON = 0  'stop timer
	INTCON = 0  'disable interupts
	PIR1 = 0
	PIE1 = 0
	myb2 = 0  'clear counters
	myb1 = 0  'clear counters
	myb0 = 0  'clear counters
	TMR1H = 0
	TMR1L = 0
	T1CON = 0x06  'set for T1CLK in for async counter no prescale, gate currently shut
ASM:        movlw 0x1F  'Set to bank 00
ASM:        andwf status,f
ASM:        bsf 0x10,0  'open gate TMR1 started
ASM:d1loop2: decfsz myd1,F  'inner delay loop first time use initial value all others use 0=256
ASM:        goto mycchk2
ASM:        goto d2loop2
ASM:mycchk2: btfss 0x0c,0  'look for an overflow
ASM:        goto mychkf2
ASM:        bcf 0x0c,0  'clear the overflow
ASM:        incf myb2,F  'increment the overflow counter
ASM:        nop
ASM:        goto d1loop2
ASM:mychkf2: nop  'not OF so equalize delay
ASM:        nop
ASM:        goto d1loop2
ASM:d2loop2: decfsz myd2,F  'second level delay loop first time use initial value all others use 0=256
ASM:        goto d1loop2
ASM:        decfsz myd3,F  'outer delay loop uses initial value
ASM:        goto d1loop2
ASM:        nop  'fine tuning
ASM:        nop
ASM:        bcf 0x10,0  'close gate, counter stopped
ASM:        btfss 0x0c,0  'check for a count/OF coming in while busy elsewhere
ASM:        goto doneit2
ASM:        incf myb2,F
ASM:doneit2: bcf 0x0c,0
myb1 = TMR1H
myb0 = TMR1L
Return                                            
sendbyte:  'DDS style send bits in byte myparam
	For mycnt2 = 0 To 7  'for all bits in byte
		gend2 = myparam.0  'set data to bit 0 of byte to send
		gend1 = 1  'toggle clock line
		WaitUs 10  'wait a bit
		gend1 = 0
		WaitUs 10
		myparam = ShiftRight(myparam, 1)  'move next bit down
	Next mycnt2
Return                                            
adddb:  'Add DiBits to store from mydbit
Select Case tempnxt
	Case 0
		tempstr.0 = mydbit.0
		tempstr.1 = mydbit.1
		tempnxt = 2
	Case 2
		tempstr.2 = mydbit.0
		tempstr.3 = mydbit.1
		tempnxt = 4
	Case 4
		tempstr.4 = mydbit.0
		tempstr.5 = mydbit.1
		tempnxt = 6
	Case 6
		tempstr.6 = mydbit.0
		tempstr.7 = mydbit.1
		tempnxt = 0
		myparam = tempstr
		tempstr = 0
		If bptr <> 9 Then
			Gosub storeit
			Gosub incptr
		Endif
EndSelect
Return                                            
addbyte:  'Add byte to store from mybyte

	mydbit.0 = mybyte.0
	mydbit.1 = mybyte.1
	Gosub adddb
	mydbit.0 = mybyte.2
	mydbit.1 = mybyte.3
	Gosub adddb
	mydbit.0 = mybyte.4
	mydbit.1 = mybyte.5
	Gosub adddb
	mydbit.0 = mybyte.6
	mydbit.1 = mybyte.7
	Gosub adddb

Return                                            
getdb:  'Get DiBits from store to mydbit
Select Case tempnxt
	Case 0
		If bptr <> 9 Then
			Gosub getitback
			Gosub incptr
		Endif
		tempstr = myparam
		mydbit.0 = tempstr.0
		mydbit.1 = tempstr.1
		tempnxt = 2
	Case 2
		mydbit.0 = tempstr.2
		mydbit.1 = tempstr.3
		tempnxt = 4
	Case 4
		mydbit.0 = tempstr.4
		mydbit.1 = tempstr.5
		tempnxt = 6
	Case 6
		mydbit.0 = tempstr.6
		mydbit.1 = tempstr.7
		tempnxt = 0
EndSelect
Return                                            
getbyte:  'Get byte from store to mybyte
	Gosub getdb
	mybyte.0 = mydbit.0
	mybyte.1 = mydbit.1
	Gosub getdb
	mybyte.2 = mydbit.0
	mybyte.3 = mydbit.1
	Gosub getdb
	mybyte.4 = mydbit.0
	mybyte.5 = mydbit.1
	Gosub getdb
	mybyte.6 = mydbit.0
	mybyte.7 = mydbit.1
Return                                            
incptr:  'increment the block address and pointer as necessary if full bptr = 9
If aptr = mymax(bptr) Then
	If bptr = 3 Then
		'all full set pointer to 9
		bptr = 9
	Else
		bptr = bptr + 1  'set to stat of next block
	Endif
	aptr = 0
Else
	aptr = aptr + 1
Endif
Return                                            
storeit:  'Add the byte in myparam to the store at next address
Select Case bptr
	Case 0
		mydata(aptr) = myparam
	Case 1
		mydata1(aptr) = myparam
	Case 2
		mydata2(aptr) = myparam
	Case 3
		mydata3(aptr) = myparam
EndSelect
Return                                            
getitback:  'get the byte in the store at next address to myparam
Select Case bptr
	Case 0
		myparam = mydata(aptr)
	Case 1
		myparam = mydata1(aptr)
	Case 2
		myparam = mydata2(aptr)
	Case 3
		myparam = mydata3(aptr)
EndSelect
Return                                            